# -*- coding: utf-8 -*-

#  PycaWM
#  placement.py
#  Copyright (c) 2007 Vincent Rasneur, Anaël Verrier

#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; version 3 only.

#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import sys

if sys.version_info < (2, 5):
    raise ImportError('placement module needs at least Python 2.5!')

from client import Client

class Placement(object):
    def place_client(self, wm, client):
        return False

class SmartPlacement(Placement):
    def place_client(self, wm, client):
        clients = filter(lambda c: c != client, wm.stacked_clients)
        for y in range(0, wm.screen.height_in_pixels - client.geometry.height,
                       10):
            x = 0
            while x < (wm.screen.width_in_pixels -
                       (client.geometry.width +
                        (client.parent_border_width * 2))):
                new_x = None
                for client_ in clients:
                    client.update_geometry(x=x, y=y)
                    if not client_.is_hidden() and client_.collide(client):
                        new_x = (client_.geometry.x + client_.geometry.width +
                                 (client_.parent_border_width * 2))

                if new_x is None:
                    client.move(dict(x=x, y=y))
                    return True

                if new_x > x:
                    x = new_x
                else:
                    x += 10

        return False

class CascadePlacement(Placement):
    x=0
    y=0

    @classmethod
    def update_coords(cls, wm, client):
        offset = Client.title_bar_height + Client.parent_border_width
        cls.x += offset
        cls.y += offset
        if (cls.x > wm.screen.width_in_pixels - 200 or
            cls.y > wm.screen.height_in_pixels - 200):
            cls.x = 0
            cls.y = 0

    def place_client(self, wm, client):
        client.move(dict(x=self.x, y=self.y))
        self.update_coords(wm, client)
        return True
